﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Windows;

namespace IncrementDecrementGUI
{
	class IncrementDecrementViewModel : INotifyPropertyChanged
	{
		IncrementDecrementModel model;

		#region INotifyPropertyChanged Implementation

		/// <summary>
		/// Das im INotifyPropertyChanged Interface definierte Event
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>
		/// Übliche virtuelle Methode, durch die Sub-Klassen das Auslösen des Events
		/// überschreiben können
		/// </summary>
		/// <param name="e">Argumente des PropertyChanged-Events</param>
		protected void OnPropertyChanged(PropertyChangedEventArgs e)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, e);
			}
		}

		/// <summary>
		/// Hilfsmethode, um das PropertyChanged Event auszulösen 
		/// </summary>
		/// <param name="propertyName">Der Name der geänderten Property</param>
		protected void NotifyPropertyChanged(string propertyName)
		{
			PropertyChangedEventArgs e = new PropertyChangedEventArgs(propertyName);
			OnPropertyChanged(e);
		}

		#endregion

		#region UserFeedback

		/// <summary>
		/// User-Feedbaack-Event
		/// </summary>
		public event EventHandler<EventArgs> UserFeedback;

		protected virtual void OnUserFeedback(EventArgs e)
		{
			if (UserFeedback == null) throw new InvalidOperationException("UserFeedbackRequired event must have a listener.");
			UserFeedback(this, e);
		}

		protected void RaiseUserFeedback(EventArgs e)
		{
			OnUserFeedback(e);
		}

		#endregion
		
		/// <summary>
		/// Konstruktor
		/// </summary>
		public IncrementDecrementViewModel()
		{
			model = new IncrementDecrementModel();
		}

		/// <summary>
		/// Ruft den Wert der Textbox ab oder legt diesen fest.
		/// </summary>
		public string TextBoxValue
		{
			get 
			{ 
				return model.TextBoxValue.ToString(); 
			}
			set 
			{
				try
				{
					int tmp = Convert.ToInt32(value);
					model.TextBoxValue = Convert.ToInt32(value);
					NotifyPropertyChanged("TextBoxValue");
				}
				catch (Exception)
				{
					RaiseUserFeedback(new EventArgs());
					model.TextBoxValue -= 1;
					NotifyPropertyChanged("TextBoxValue");
					model.TextBoxValue += 1;
					NotifyPropertyChanged("TextBoxValue");
				}
			}
		}

		/// <summary>
		/// Ruft den Wert der Fehler-Textbox ab oder legt diesen fest.
		/// </summary>
		public string ErrorTextValue
		{
			get
			{
				return model.ErrorTextValue;
			}
			set
			{
				model.ErrorTextValue = value;
			}
		}

		public void IncrementTextBoxValue()
		{
			model.TextBoxValue++;
			NotifyPropertyChanged("TextBoxValue");
		}

		public void DecrementTextBoxValue()
		{
			model.TextBoxValue--;
			NotifyPropertyChanged("TextBoxValue");
		}
	}
}
